@@ -3,12 +3,13 @@ import numpy as np
 import pandas as pd
 import statsmodels.formula.api as smf
 import matplotlib.pyplot as plt
-from scipy.stats import norm
+from scipy.stats import ncf, f, norm
 import scipy.stats as stats
 import statsmodels.stats.multitest as multi
 from statsmodels.sandbox.regression.predstd import wls_prediction_std
 import statsmodels.api as sm
 
+
 def prepare_df(X1, Y1, X2, Y2):
     H1 = np.zeros(X1.size)
     H2 = np.ones(X2.size)
@@ -138,7 +139,7 @@ def plot_single(data, results, test='', plot_measurements=True, save_to='', plot
     color = "black"
         
     if plot_dense:
-        X_fit = np.linspace(min(X), max(X), 100)
+        X_fit = np.linspace(min(X), 100, 1000)
         
         rrr_fit= np.cos(2*np.pi*X_fit/period)
         sss_fit = np.sin(2*np.pi*X_fit/period)
@@ -175,7 +176,12 @@ def plot_single(data, results, test='', plot_measurements=True, save_to='', plot
     
     plt.plot(X_fit, Y_fit, color=color, label = test)    
     
-    plt.title(test+" p-value="+str(results.f_pvalue)+", period="+str(period))
+    if plot_measurements:
+        plt.axis([min(min(X),0), 1.1*max(max(X),period), 0.9*min(min(Y), min(Y_fit)), 1.1*max(max(Y), max(Y_fit))])
+    else:
+        plt.axis([min(X_fit), 50, min(Y_fit)*0.9, max(Y_fit)*1.1])
+        
+    plt.title(test + ', p-value=' + "{0:.5f}".format(results.f_pvalue))
     
     if save_to:
         plt.savefig(save_to+'.pdf')
@@ -895,7 +901,7 @@ def test_cosinor_pair(data, period):
     ind_Z_acr = diff_est_acr/np.sqrt(np.diag(diff_var_acr))
     interval_acr = np.array((diff_est_acr, diff_est_acr - 1.96 * np.sqrt(np.diag(diff_var_acr)), diff_est_acr + 1.96 * np.sqrt(np.diag(diff_var_acr))))
     df_acr = 1
-    global_p_value_acr = 1-stats.chi2.cdf(glob_chi_acr, df_amp)
+    global_p_value_acr = 1-stats.chi2.cdf(glob_chi_acr, df_acr)
     ind_p_value_acr = 2*norm.cdf(-abs(ind_Z_acr))
     
     
@@ -923,4 +929,55 @@ def test_cosinor_pair(data, period):
     
     return results, statistics_raw, statistics_trans, global_test_amp, ind_test_amp, global_test_acr, ind_test_acr
                     
-    
\ No newline at end of file
+
+# amplitude_detection: determines the minimal number of samples to obtain a statistically significant result for the zero amplitude test
+# A ... presumed amplitude
+# var ... residual variance
+# p ... threshold for statistical significance of the zero amplitude test
+# alpha ... type I error probability
+def amplitude_detection(A, var, p = 0.95, alpha = 0.05):
+
+    sigma2 = var
+    C_2 = A**2/sigma2 
+
+    N = 4
+    
+    while True:
+        lmbda = (N * C_2)/4       
+        f2 = f(2, N-3, 0).ppf(1-alpha)
+        F = 1-ncf(2,N-3,lmbda).cdf(f2)
+        if F >= p:
+            break
+        
+        N += 1
+        
+    return N
+
+# amplitude_confidence: determines the minimal number of samples to obtain a given length of the confidence interval for the estimated amplitude
+# L ... maximal acceptable length of the confidence interval
+# var ... residual variance
+# alpha ... 1-alpha = confidence level of the cofidence interval
+def amplitude_confidence(L, var, alpha = 0.05):
+    sigma2 = var
+    N = 8 * sigma2 * (norm.ppf(1-alpha/2)**2)/(L**2)
+    return int(np.ceil(N))
+
+# acrophase_confidence: determines the minimal number of samples to obtain a given length of the confidence interval for the estimated acrophase
+# L ... maximal acceptable length of the confidence interval
+# A_0 ... presumed minimal amplitude
+# var ... residual variance
+# alpha ... 1-alpha = confidence level of the cofidence interval
+def acrophase_confidence(L, A_0, var, alpha = 0.05):
+    sigma2 = var
+    #N = 8 * sigma2 * (norm.ppf(1-alpha/2)**2)/(L**2 * A_0**2) # approximation
+    N = (2 * norm.ppf(1-alpha/2)**2 * sigma2)/(A_0**2 * np.sin(L/2)**2) # accurate
+
+    return int(np.ceil(N))
+
+# acrophase_shift_detection: determines the minimal number of samples to detect a specific shift in the acrophase
+def acrophase_shift_detection(shift, A_0, var, alpha = 0.05):
+    L = 0.5*shift
+    return acrophase_confidence(L, A_0, var, alpha)    
+
+    
+
